@@ -1,11 +1,11 @@
 import warnings
+from .version import __version__
 
 # Show all deprecated warning only once:
 warnings.filterwarnings('once', category=DeprecationWarning)
 del warnings
 
-from rconn import rconnect, connect
-from taggedContainers import TaggedList
+from .rconn import connect
+from .taggedContainers import TaggedList, TaggedArray, AttrArray
 
-__version__ = '0.5.2'
 
