@@ -60,7 +60,7 @@ class TaggedList(object):
                 
     def astuples(self):
         """convert a TaggedList into a representation suitable to be provided to __init__()"""
-        return zip(self.keys, self.values)
+        return list(zip(self.keys, self.values))
         
     def __repr__(self): 
         data = ["%s=%s" % (key, repr(value)) if key else "'%s'" % value for key,value in self.astuples()]
@@ -78,6 +78,14 @@ class TaggedList(object):
 #    def __cmp__(self, other):
 #        return cmp(self.values, self.__cast(other))
     __hash__ = None # Mutable sequence, so not hashable
+
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return False
+        return self.__dict__ == other.__dict__
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
     
     def __contains__(self, item): 
         return item in self.values
@@ -236,11 +244,22 @@ class AttrArray(numpy.ndarray):
         if hasattr(self, 'attr'):
             return r[:-1] + ', attr=' + repr(self.attr) + ')'
         return r
+        
+    @classmethod
+    def new(cls, data, attr):
+        """Factory method to create AttrArray objects from ndarrays or Python lists.
+        Usage: AttrArray.new(array([1, 2, 3, 4]), {'attr1': val1, 'attr2': val2})
+        """
+        if not isinstance(data, numpy.ndarray):
+	    arr = numpy.array(data)  # assume it is a Python list or any other valid data type for arrays
+	else:
+	    arr = data 
+        attrArr = arr.view(AttrArray)
+        attrArr.attr = attr
+        return attrArr
 
-def asAttrArray(ndarray, attr):
-    arr = ndarray.view(AttrArray)
-    arr.attr = attr
-    return arr
+def asAttrArray(data, attr):
+    return AttrArray.new(data, attr)
 
 
 class TaggedArray(AttrArray):
@@ -285,20 +304,29 @@ class TaggedArray(AttrArray):
     def keys(self):
         return self.attr[:]
 
+    @classmethod
+    def new(cls, data, tags):
+        """Factory method to create TaggedArray objects from ndarrays or Python lists.
+        Check the docs in TaggedArray for more information.
+        Usage:
+        l = TaggedArray.new(array([1, 2, 3, 4]), ['v1', 'v2', 'v3', 'v4'])
+        l[0]     # returns 1
+        l['v1']  # returns 1
+        l['v2']  # returns 2  (not 4 !)
+        l[3]     # returns 4
+        """
+        if len(tags) != len(data):
+            raise ValueError('Number of keys must match size of array')
+        if not isinstance(data, numpy.ndarray):
+	    arr = numpy.array(data)  # assume it is a Python list or any other valid data type for arrays
+	else:
+	    arr = data 
+	
+        taggedArr = arr.view(cls)
+        taggedArr.attr = tags
+        return taggedArr
 
-def asTaggedArray(ndarray, tags):
-    """Factory function to create TaggedArray objects. Check the docs in TaggedArray for more information.
 
-    Usage:
-    l = asTaggedArray(array([1, 2, 3, 4]), ['v1', 'v2', 'v3', 'v4'])
-    l[0]     # returns 1
-    l['v1']  # returns 1
-    l['v2']  # returns 2  (not 4 !)
-    l[3]     # returns 4
-    """
-    if len(tags) != len(ndarray):
-        raise ValueError('Number of keys must match size of array')
-    arr = ndarray.view(TaggedArray)
-    arr.attr = tags
-    return arr
-    
+def asTaggedArray(data, tags):
+    return TaggedArray.new(data, tags)
+
